function clearOldWindows() {
  var current_data = new Promise(function(resolve, reject) {
    chrome.storage.local.get(function(stored) {
      resolve(stored);
    });
  });

  var current_windows = new Promise(function(resolve, reject) {
    chrome.windows.getAll({populate: true}, function(windows) {
      resolve(windows);
    });
  });

  Promise.all([current_data, current_windows]).then(function(resolved_data) {
    var lost_pile = {},
        new_save = {};

    // Saved content
    for (var i in resolved_data[0]) {
      // Window id
      if (i == parseInt(i)) {
        for (var j = 0; j < resolved_data[1].length; j++) {
          // Found a value
          (function(saved_window_id, saved_window_obj, window_obj, lost_pile) {
            var out_of_band = null;

            if (saved_window_obj.tabs.length != window_obj.tabs.length) {
              lost_pile[saved_window_id] = true;
              return;
            }

            for (var t = 0; t < window_obj.tabs.length; t++) {
              if (saved_window_obj.tabs[t].url != window_obj.tabs[t].url) {
                lost_pile[saved_window_id];
                return;
              }
            }
            // If we are here, we found a match.
            chrome.storage.local.remove(saved_window_id);
            new_save[saved_window_id] = saved_window_obj;
            chrome.storage.local.set(new_save);
          })(i, resolved_data[0][i], resolved_data[1][j], lost_pile);
        }
      }
    }

    for (var window_id_key in lost_pile) {
      chrome.storage.local.remove(window_id_key.toString());
    }
  });
}

window.onbeforeunload = function(e) {
  e.preventDefault();
  clearOldWindows();
};

setInterval(function() {
  chrome.storage.local.get(function(data) {
    chrome.windows.getAll(function(windows) {
      for (var key in data) {
        if (key == parseInt(key)) {
          for (var window_idx = 0; window_idx < windows.length; window_idx++) {
            if (key == windows[window_idx].id) {
              data['UltiTab-Window'][data[key].name].position = {
                top: windows[window_idx].top,
                left: windows[window_idx].left,
                width: windows[window_idx].width,
                height: windows[window_idx].height,
                state: windows[window_idx].state
              };
            }
          }
        }
      }
      chrome.storage.local.set({"UltiTab-Window": data['UltiTab-Window']});
    });
  });
}, 2500);

// Real window management
chrome.windows.onRemoved.addListener(function(windowId) {
  chrome.storage.local.remove(windowId.toString());
});

chrome.tabs.onRemoved.addListener(function(tab, removedInfo) {
  if (!removedInfo.isWindowClosing) {
    setTimeout(function() {
      removeTabs(removedInfo.windowId);
    }, 1200);
  } else {
    chrome.storage.local.remove(removedInfo.windowId.toString());
  }
});

chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
  syncTabs(tab.windowId);
});

chrome.tabs.onDetached.addListener(function(tabId, changeInfo) {
  syncTabs(changeInfo.oldWindowId);
});

chrome.tabs.onAttached.addListener(function(tabId, changedInfo) {
  syncTabs(changedInfo.newWindowId);
});

buildContextMenu();

chrome.runtime.onMessage.addListener(
  function(request, sender, sendResponse) {
    if (request.type == 'create_window') {
      backgroundCreateWindow(
        request.content.current_window,
        request.content.window_item,
        request.content.name
      );
    } else if (request.type == 'change_browser_icon') {
      if (request.content.browser_action_icon === 0) {
        chrome.browserAction.setIcon({path:"vaux_128.png"});
      } else {
        chrome.browserAction.setIcon({path:"vaux_logo_dark_theme.png"});
      }
    }
  }
);

chrome.runtime.onUpdateAvailable.addListener(function() {
  chrome.runtime.reload();
});

chrome.storage.local.get("customize", function(custom_data) {
  if (typeof custom_data.customize === 'undefined' || custom_data.customize.browser_action_icon === 0) {
    chrome.browserAction.setIcon({path:"vaux_128.png"});
  } else {
    chrome.browserAction.setIcon({path:"vaux_logo_dark_theme.png"});
  }
});