chrome.storage.local.get('instance_id', function (instance) {

	if (typeof instance.instance_id == 'undefined') {
		var instance_seed = new Uint8Array(32);

		crypto.getRandomValues(instance_seed);

		var running_id = '';

		for (var i = 0; i < instance_seed.length; ++i) {
			running_id = running_id + instance_seed[i].toString(16);
		}

		instance.instance_id = running_id;
	}

	chrome.storage.local.set({ 'instance_id': instance.instance_id });
});

function genericOnClick(info, tab) {
	chrome.storage.local.get(function (data) {
		for (var key in data) {
			if (key == parseInt(key)) {
				if (data[key].name == info.menuItemId) {
					chrome.tabs.move(tab.id, { windowId: parseInt(key), index: -1 });
					_gaq.push(['_trackEvent', 'context_menus', 'moved to open']);
					return;
				}
			}
		}

		for (var window_name in data['UltiTab-Window']) {
			if (info.menuItemId == window_name) {
				if (data['UltiTab-Window'][window_name].read_only) {
					return;
				}

				data['UltiTab-Window'][window_name].tabs.push(tab);

				chrome.storage.local.set({ 'UltiTab-Window': data['UltiTab-Window'] }, function () {
					chrome.storage.local.get("UltiTab-Window", function (current_data) {
						var current_data_tags = current_data['UltiTab-Window'][info.menuItemId].tabs;

						if (current_data_tags[current_data_tags.length - 1].url == tab.url) {
							chrome.tabs.remove(tab.id);
							_gaq.push(['_trackEvent', 'context_menus', 'moved to saved']);
						}

					});
				});

				break;
			}
		}
	});
}

function buildContextMenu() {
	chrome.contextMenus.removeAll(function () {
		chrome.storage.local.get('UltiTab-Window', function (data) {
			var page_parent = chrome.contextMenus.create({
				"id": 'page_parent',
				"title": "Send tab to saved window:",
				"contexts": ['page']
			}
			);

			for (var tab_name in data['UltiTab-Window']) {
				var title = tab_name;

				if (data['UltiTab-Window'][tab_name].read_only) {
					title = title + ' (Autosave Paused)';
				}

				chrome.contextMenus.create({ "id": tab_name, "title": title, "parentId": page_parent });
			}
		});
	});
}

chrome.contextMenus.onClicked.addListener(genericOnClick);

function backgroundCreateWindow(
	current_window,
	window_item,
	window_item_text
) {
	if (current_window.tabs.length == 1 &&
		current_window.tabs[0].url.indexOf('chrome://newtab') == 0 &&
		current_window.incognito == window_item.window_info.incognito
	) {

		if (window_item.window_info.position) {
			chrome.windows.update(current_window.id, {
				top: window_item.window_info.position.top,
				left: window_item.window_info.position.left,
				width: window_item.window_info.position.width,
				height: window_item.window_info.position.height,
				state: window_item.window_info.position.state
			});
		}

		chrome.tabs.update(current_window.tabs[0].id,
			{
				url: window_item.window_info.tabs[0].url,
				pinned: window_item.window_info.tabs[0].pinned,
				muted: window_item.window_info.tabs[0].muted,
				active: window_item.window_info.tabs[0].active
			},
			function () {
				for (var i = 1; i < window_item.window_info.tabs.length; i++) {
					chrome.tabs.create({
						windowId: current_window.id,
						url: window_item.window_info.tabs[i].url,
						active: window_item.window_info.tabs[i].active,
						pinned: window_item.window_info.tabs[i].pinned
					});
				}

				setRunningWindowInfo(
					current_window.id,
					window_item_text,
					'',
					window_item.window_info.tabs,
					current_window.incognito,
					{},
					function () { },
					function () { }
				);
			}
		);
	} else {
		var first_url = 'chrome://newtab/';

		if (window_item.window_info.tabs.length > 0) {
			first_url = (typeof window_item.window_info.tabs[0].url != 'undefined' ? window_item.window_info.tabs[0].url : window_item.window_info.tabs[0]);
		}

		var create_obj = {
			url: [first_url],
			incognito: (window_item.window_info.incognito ? window_item.window_info.incognito : false)
		}

		if (window_item.window_info.position) {
			create_obj['top'] = window_item.window_info.position.top;
			create_obj['left'] = window_item.window_info.position.left;
			create_obj['width'] = window_item.window_info.position.width;
			create_obj['height'] = window_item.window_info.position.height;
		}

		chrome.windows.create(
			create_obj,
			function (window_obj) {

				if (typeof window_item.window_info.position != 'undefined' &&
					typeof window_item.window_info.position.state != 'undefined' &&
					window_obj.state != window_item.window_info.position.state
				) {
					chrome.windows.update(window_obj.id, { state: window_item.window_info.position.state });
				}

				chrome.tabs.update(
					window_obj.tabs[0].id,
					{
						pinned: window_item.window_info.tabs[0].pinned,
						muted: window_item.window_info.tabs[0].muted,
						active: window_item.window_info.tabs[0].active
					},
					function () {
						for (var i = 1; i < window_item.window_info.tabs.length; i++) {
							window_item.window_info.tabs[i].windowId = window_obj.id;
							chrome.tabs.create(
								{
									windowId: window_obj.id,
									url: window_item.window_info.tabs[i].url,
									active: window_item.window_info.tabs[i].active,
									pinned: window_item.window_info.tabs[i].pinned
								},
								function (tab) {
									if (!window_item.window_info.tabs[i].active) {
										chrome.tabs.discard(tab.id);
									}
								}
							);
						}

						setRunningWindowInfo(
							window_obj.id,
							window_item_text,
							'',
							window_item.window_info.tabs,
							window_obj.incognito,
							{},
							function () { },
							function () { }
						);
					}
				);
			}
		);
	}
}

function setRunningWindowInfo(
	window_id,
	new_name,
	old_name,
	tabs,
	incognito,
	sets,
	callback,
	error_callback,
	running_windows
) {
	updateSavedWindow(
		new_name,
		old_name,
		tabs,
		incognito,
		sets,
		function (sets) {
			var window_save = {};
			window_save[window_id.toString()] = { name: new_name, tabs: tabs };

			// Overwrite values of current window by id
			chrome.storage.local.set(
				window_save,
				function () {
					if (typeof running_windows != 'undefined') {
						// Change out running_windows
						if (running_windows[old_name]) {
							delete running_windows[old_name];
						}

						running_windows[new_name] = window_id;
					}

					callback(sets);
				}
			);
		},
		error_callback
	);
}

function updateSavedWindow(
	new_name,
	old_name,
	tabs,
	incognito,
	sets,
	callback,
	error_callback
) {
	chrome.storage.local.get(
		'UltiTab-Window',
		function (current_save) {
			var read_only = false,
				lock = false,
				position = {},
				saved_window_url = false;

			if (typeof current_save['UltiTab-Window'] == 'undefined') {
				current_save['UltiTab-Window'] = {};
			}

			if (typeof current_save["UltiTab-Window"][old_name] != 'undefined') {
				if (typeof current_save["UltiTab-Window"][new_name] != 'undefined') {
					error_callback();
					return;
				}

				read_only = current_save["UltiTab-Window"][old_name].read_only;
				lock = current_save["UltiTab-Window"][old_name].lock;
				position = current_save["UltiTab-Window"][old_name].position;

				if (read_only) {
					tabs = current_save["UltiTab-Window"][old_name].tabs;
				}

				if (typeof current_save['UltiTab-Window'][old_name].saved_window_url != 'undefined') {
					saved_window_url = current_save['UltiTab-Window'][old_name].saved_window_url;
				}

				delete current_save["UltiTab-Window"][old_name];
			}

			if (typeof current_save["UltiTab-Window"][new_name] != 'undefined') {
				if (old_name.trim() != '') {
					error_callback();
					return;
				}

				read_only = current_save["UltiTab-Window"][new_name].read_only;
				lock = current_save["UltiTab-Window"][new_name].lock;
				position = current_save["UltiTab-Window"][new_name].position;

				if (read_only) {
					tabs = current_save["UltiTab-Window"][new_name].tabs;
				}

				if (typeof current_save['UltiTab-Window'][new_name].saved_window_url != 'undefined') {
					saved_window_url = current_save['UltiTab-Window'][new_name].saved_window_url;
				}
			}

			current_save["UltiTab-Window"][new_name] = {
				tabs: tabs,
				incognito: incognito,
				read_only: read_only,
				lock: lock,
				position: position
			};

			if (saved_window_url) {
				current_save["UltiTab-Window"][new_name].saved_window_url = saved_window_url;
			}

			chrome.storage.local.set(
				{ "UltiTab-Window": current_save["UltiTab-Window"] },
				function () {
					sets = { "UltiTab-Window": current_save["UltiTab-Window"] };
					buildContextMenu();
					callback(sets);
				}
			);
		}
	);
}

function clearOldWindows() {
	var current_data = new Promise(function (resolve, reject) {
		chrome.storage.local.get(function (stored) {
			resolve(stored);
		});
	});

	var current_windows = new Promise(function (resolve, reject) {
		chrome.windows.getAll({ populate: true }, function (windows) {
			resolve(windows);
		});
	});

	Promise.all([current_data, current_windows]).then(function (resolved_data) {
		var lost_pile = {},
			new_save = {};

		// Saved content
		for (var i in resolved_data[0]) {
			// Window id
			if (i == parseInt(i)) {
				for (var j = 0; j < resolved_data[1].length; j++) {
					// Found a value
					(function (saved_window_id, saved_window_obj, window_obj, lost_pile) {
						var out_of_band = null;

						if (saved_window_obj.tabs.length != window_obj.tabs.length) {
							lost_pile[saved_window_id] = true;
							return;
						}

						for (var t = 0; t < window_obj.tabs.length; t++) {
							if (saved_window_obj.tabs[t].url != window_obj.tabs[t].url) {
								lost_pile[saved_window_id];
								return;
							}
						}
						// If we are here, we found a match.
						chrome.storage.local.remove(saved_window_id);
						new_save[saved_window_id] = saved_window_obj;
						chrome.storage.local.set(new_save);
					})(i, resolved_data[0][i], resolved_data[1][j], lost_pile);
				}
			}
		}

		for (var window_id_key in lost_pile) {
			chrome.storage.local.remove(window_id_key.toString());
		}
	});
}

setInterval(function () {
	chrome.storage.local.get(function (data) {
		chrome.windows.getAll(function (windows) {
			for (var key in data) {
				if (key == parseInt(key)) {
					for (var window_idx = 0; window_idx < windows.length; window_idx++) {
						if (key == windows[window_idx].id) {
							data['UltiTab-Window'][data[key].name].position = {
								top: windows[window_idx].top,
								left: windows[window_idx].left,
								width: windows[window_idx].width,
								height: windows[window_idx].height,
								state: windows[window_idx].state
							};
						}
					}
				}
			}
			chrome.storage.local.set({ "UltiTab-Window": data['UltiTab-Window'] });
		});
	});
}, 2500);

// Real window management
chrome.windows.onRemoved.addListener(function (windowId) {
	chrome.storage.local.remove(windowId.toString());
});

chrome.tabs.onRemoved.addListener(function (tab, removedInfo) {
	if (!removedInfo.isWindowClosing) {
		setTimeout(function () {
			removeTabs(removedInfo.windowId);
		}, 1200);
	} else {
		chrome.storage.local.remove(removedInfo.windowId.toString());
	}
});

chrome.tabs.onUpdated.addListener(function (tabId, changeInfo, tab) {
	syncTabs(tab.windowId);
});

chrome.tabs.onDetached.addListener(function (tabId, changeInfo) {
	syncTabs(changeInfo.oldWindowId);
});

chrome.tabs.onAttached.addListener(function (tabId, changedInfo) {
	syncTabs(changedInfo.newWindowId);
});

buildContextMenu();

chrome.runtime.onMessage.addListener(
	function (request, sender, sendResponse) {
		if (request.type == 'create_window') {
			backgroundCreateWindow(
				request.content.current_window,
				request.content.window_item,
				request.content.name
			);

			sendResponse(true);

		} else if (request.type == 'change_browser_icon') {
			if (request.content.browser_action_icon === 0) {
				chrome.action.setIcon({ path: "vaux_128.png" });
			} else {
				chrome.action.setIcon({ path: "vaux_logo_dark_theme.png" });
			}
		}
	}
);

chrome.runtime.onUpdateAvailable.addListener(function () {
	chrome.runtime.reload();
});

chrome.storage.local.get("customize", function (custom_data) {
	if (typeof custom_data.customize === 'undefined' || custom_data.customize.browser_action_icon === 0) {
		chrome.action.setIcon({ path: "vaux_128.png" });
	} else {
		chrome.action.setIcon({ path: "vaux_logo_dark_theme.png" });
	}
});

function syncTabs(windowId) {
	chrome.storage.local.get(windowId.toString(), function (data) {
		if (typeof data[windowId.toString()] != 'undefined' &&
			typeof data[windowId.toString()].name != 'undefined'
		) {
			try {
				chrome.windows.get(windowId, { populate: true }, function (windowInfo) {
					if (chrome.runtime.lastError) {
						return;
					}

					chrome.storage.local.get('UltiTab-Window', function (windows) {
						if (windows['UltiTab-Window'][data[windowId.toString()].name] &&
							windowInfo
						) {
							if (windows['UltiTab-Window'][data[windowId.toString()].name].read_only) {
								return true;
							}

							windows['UltiTab-Window'][data[windowId.toString()].name].tabs = windowInfo.tabs;
							windows['UltiTab-Window'][data[windowId.toString()].name].incognito = windowInfo.incognito;
							windows['UltiTab-Window'][data[windowId.toString()].name].read_only = false;
							windows['UltiTab-Window'][data[windowId.toString()].name].position = {
								top: windowInfo.top,
								left: windowInfo.left,
								width: windowInfo.width,
								height: windowInfo.height,
								state: windowInfo.state
							};

							chrome.storage.local.set({
								"UltiTab-Window": windows['UltiTab-Window']
							},
								function () { }
							);
						}
					});
				});
			} catch (e) { }
		}
	});
}

function removeTabs(windowId) {
	chrome.windows.get(windowId, function (windowInfo) {
		if (chrome.runtime.lastError) {
			return;
		}

		syncTabs(windowId);
	});
}