function backgroundCreateWindow(
  current_window,
  window_item,
  window_item_text
) {
  if (current_window.tabs.length == 1 &&
    current_window.tabs[0].url.indexOf('chrome://newtab') == 0 &&
    current_window.incognito == window_item.window_info.incognito
  ) {

    if (window_item.window_info.position) {
      chrome.windows.update(current_window.id, {
        top: window_item.window_info.position.top,
        left: window_item.window_info.position.left,
        width: window_item.window_info.position.width,
        height: window_item.window_info.position.height,
        state: window_item.window_info.position.state
      });
    }

    chrome.tabs.update(current_window.tabs[0].id,
      {
        url: window_item.window_info.tabs[0].url,
        pinned: window_item.window_info.tabs[0].pinned,
        muted: window_item.window_info.tabs[0].muted,
        active: window_item.window_info.tabs[0].active
      },
      function() {
          for (var i = 1; i < window_item.window_info.tabs.length; i++) {
            chrome.tabs.create({
              windowId: current_window.id,
              url: window_item.window_info.tabs[i].url,
              active: window_item.window_info.tabs[i].active,
              pinned: window_item.window_info.tabs[i].pinned
            });
          }

          setRunningWindowInfo(
            current_window.id,
            window_item_text,
            '',
            window_item.window_info.tabs,
            current_window.incognito,
            {},
            function() {},
            function() {}
          );
      }
    );
  } else {
    var first_url = 'chrome://newtab/';

    if (window_item.window_info.tabs.length > 0) {
      first_url = (typeof window_item.window_info.tabs[0].url != 'undefined' ? window_item.window_info.tabs[0].url : window_item.window_info.tabs[0]);
    }

    var create_obj = {
      url: [first_url],
      incognito: (window_item.window_info.incognito ? window_item.window_info.incognito : false)
    }

    if (window_item.window_info.position) {
      create_obj['top'] = window_item.window_info.position.top;
      create_obj['left'] = window_item.window_info.position.left;
      create_obj['width'] = window_item.window_info.position.width;
      create_obj['height'] = window_item.window_info.position.height;
    }

    chrome.windows.create(
      create_obj,
      function(window_obj) {

        if (typeof window_item.window_info.position != 'undefined' &&
            typeof window_item.window_info.position.state != 'undefined' &&
            window_obj.state != window_item.window_info.position.state
        ) {
          chrome.windows.update(window_obj.id, {state: window_item.window_info.position.state});
        }

        chrome.tabs.update(
          window_obj.tabs[0].id,
          {
            pinned: window_item.window_info.tabs[0].pinned,
            muted: window_item.window_info.tabs[0].muted,
            active: window_item.window_info.tabs[0].active
          },
          function() {
            for (var i = 1; i < window_item.window_info.tabs.length; i++) {
              window_item.window_info.tabs[i].windowId = window_obj.id;
              chrome.tabs.create(
                {
                  windowId: window_obj.id,
                  url: window_item.window_info.tabs[i].url,
                  active: window_item.window_info.tabs[i].active,
                  pinned: window_item.window_info.tabs[i].pinned
                },
                function(tab) {
                  if (!window_item.window_info.tabs[i].active) {
                    chrome.tabs.discard(tab.id);
                  }
                }
              );
            }

            setRunningWindowInfo(
              window_obj.id,
              window_item_text,
              '',
              window_item.window_info.tabs,
              window_obj.incognito,
              {},
              function() {},
              function() {}
            );
          }
        );
      }
    );
  }
}

function setRunningWindowInfo(
  window_id,
  new_name,
  old_name,
  tabs,
  incognito,
  sets,
  callback,
  error_callback,
  running_windows
) {
  updateSavedWindow(
    new_name,
    old_name,
    tabs,
    incognito,
    sets,
    function(sets) {
      var window_save = {};
      window_save[window_id.toString()] = {name: new_name, tabs: tabs};

      // Overwrite values of current window by id
      chrome.storage.local.set(
        window_save,
        function() {
          if (typeof running_windows != 'undefined') {
            // Change out running_windows
            if (running_windows[old_name]) {
              delete running_windows[old_name];
            }

            running_windows[new_name] = window_id;
          }

          callback(sets);
        }
      );
    },
    error_callback
  );
}

function updateSavedWindow(
  new_name,
  old_name,
  tabs,
  incognito,
  sets,
  callback,
  error_callback
) {
  chrome.storage.local.get(
    'UltiTab-Window',
    function(current_save) {
      var read_only = false,
          lock = false,
          position = {},
          saved_window_url = false;

      if (typeof current_save['UltiTab-Window'] == 'undefined') {
        current_save['UltiTab-Window'] = {};
      }

      if (typeof current_save["UltiTab-Window"][old_name] != 'undefined') {
        if (typeof current_save["UltiTab-Window"][new_name] != 'undefined') {
          error_callback();
          return;
        }

        read_only = current_save["UltiTab-Window"][old_name].read_only;
        lock = current_save["UltiTab-Window"][old_name].lock;
        position = current_save["UltiTab-Window"][old_name].position;

        if (read_only) {
          tabs = current_save["UltiTab-Window"][old_name].tabs;
        }

        if (typeof current_save['UltiTab-Window'][old_name].saved_window_url != 'undefined') {
          saved_window_url = current_save['UltiTab-Window'][old_name].saved_window_url;
        }

        delete current_save["UltiTab-Window"][old_name];
      }

      if (typeof current_save["UltiTab-Window"][new_name] != 'undefined') {
        if (old_name.trim() != '') {
          error_callback();
          return;
        }

        read_only = current_save["UltiTab-Window"][new_name].read_only;
        lock = current_save["UltiTab-Window"][new_name].lock;
        position = current_save["UltiTab-Window"][new_name].position;

        if (read_only) {
          tabs = current_save["UltiTab-Window"][new_name].tabs;
        }

        if (typeof current_save['UltiTab-Window'][new_name].saved_window_url != 'undefined') {
          saved_window_url = current_save['UltiTab-Window'][new_name].saved_window_url;
        }
      }

      current_save["UltiTab-Window"][new_name] = {
        tabs: tabs,
        incognito: incognito,
        read_only: read_only,
        lock: lock,
        position: position
      };

      if (saved_window_url) {
        current_save["UltiTab-Window"][new_name].saved_window_url = saved_window_url;
      }

      chrome.storage.local.set(
        {"UltiTab-Window": current_save["UltiTab-Window"]},
        function() {
          sets = {"UltiTab-Window": current_save["UltiTab-Window"]};
          buildContextMenu();
          callback(sets);
        }
      );
    }
  );
}
