// browser action management

// Query for all open windows, create order array and move them into open first then all the rest.
// Pull together all three queries = open window, open windows, saved windows
//  -> Use promise to bind them all into one callback for continuouse use.
//  -> Start building from there although it could be the sl

var running_windows = {},
	promise_deferred = [],
	total_count = 1,
	rendered = false,
	response_timeout = null,
	response_timeout_cleaner = null,
	search_key_timeout = null,
	search_added = false,
	body_added = false,
	used_search = false,
	title_hover = false,
	notsearchable = new RegExp("notsearchable");

function buildWindowlistUI(currentWindow, windowList, sets) {
	var container = document.getElementById("windows"),
		current_selection = null,
		all_elms = null,
		visible_elms = null,
		bottom_text = document.getElementById('response_text'),
		search_box = document.getElementById('search_input'),
		option_open = document.getElementById('option_open'),
		global_edit_container = document.getElementById('global_edit_container'),
		global_edit = document.createElement('span'),
		global_edit_state = false;

	global_edit_container.innerHTML = '';

	global_edit.className = "glyphicon glyphicon-pencil icon-style edit-icon pin-window-toggle action-button";

	global_edit.addEventListener("click", function () {
		var all_edits = document.querySelectorAll(".window-item > .col-first > .glyphicon-pencil");

		global_edit_state = !global_edit_state;

		if (global_edit_state) {
			global_edit.style.color = 'black';
		} else {
			global_edit.style.color = 'grey';
		}

		// This is stupid. Do not click each element.
		for (var k = 0; k < all_edits.length; k++) {
			all_edits[k].click();
		}
	});

	global_edit_container.appendChild(global_edit);

	option_open.style.cursor = 'pointer';

	option_open.addEventListener('click', function () {
		chrome.windows.create({url: "chrome-extension://" + chrome.runtime.id + "/options.html"});
	});

	search_box.focus();
	container.innerHTML = '';

	promise_deferred.push(
		createWindowItemFromList(
			container,
			currentWindow,
			running_windows,
			sets,
			'focused-on',
			bottom_text
		)
	);

	for (var i = 0; i < windowList.length; i++) {
		if (windowList[i].id != currentWindow.id) {
			total_count++;
			promise_deferred.push(
				createWindowItemFromList(
					container,
					windowList[i],
					running_windows,
					sets,
					'running',
					bottom_text
				)
			);
		}
	}

	Promise.all(promise_deferred).then(function () {
		for (var i in sets['UltiTab-Window']) {
			if (typeof running_windows[i] == 'undefined') {
				var p_elm = windowItemContainerCreate();

				total_count++;

				(function (p_elm) {
					p_elm.window_info = sets['UltiTab-Window'][i];
					var edit_state_on = false,
						incognito = sets['UltiTab-Window'][i].incognito,
						text_span = windowItemCreate(i, sets['UltiTab-Window'][i].tabs.length),
						editor_button = editorButton(),
						save_button = saveButton(),
						name_input = nameInput(i),
						left_elm = createColumn('col-first'),
						mid_elm = createColumn('col-middle'),
						right_elm = createColumn('col-end'),
						remove_button = removeButton(i, running_windows),
						readonly_button = readOnlyButton(i, sets),
						lock_button = lockBuilder(p_elm, text_span, mid_elm, mid_elm, right_elm, name_input, remove_button),
						share_button = shareButton(p_elm, (typeof p_elm.window_info.saved_window_url != 'undefined' ? true : false));

					// Next elements for containing interactions
					p_elm.appendChild(left_elm);
					p_elm.appendChild(mid_elm);
					p_elm.appendChild(right_elm);

					editor_button.style.display = 'inline-block';
					editor_button.style.visibility = 'hidden';

					name_input.addEventListener('keyup', function (event) {
						if (event.keyCode == 13) {
							updateSavedWindow(
								name_input.value,
								text_span.innerText,
								p_elm.window_info.tabs,
								incognito,
								sets,
								function () {
									text_span.innerText = name_input.value;
									readonly_button.updateName(name_input.value);
									remove_button.updateName(name_input.value);
									lock_button.updateName(name_input.value);
									writeBottomText(bottom_text, 'Name saved.', false, response_timeout, response_timeout_cleaner);
								},
								function () {
									writeBottomText(bottom_text, 'Name already used.', true, response_timeout, response_timeout_cleaner);
								}
							);
						}
					});

					save_button.addEventListener("click", function () {
						updateSavedWindow(
							name_input.value,
							text_span.innerText,
							p_elm.window_info.tabs,
							incognito,
							sets,
							function () {
								text_span.innerText = name_input.value;
								readonly_button.updateName(name_input.value);
								remove_button.updateName(name_input.value);
								lock_button.updateName(name_input.value);
								writeBottomText(bottom_text, 'Name saved.', false, response_timeout, response_timeout_cleaner);
							},
							function () {
								writeBottomText(bottom_text, 'Name already used.', true, response_timeout, response_timeout_cleaner);
							}
						);
					});

					var click_handler = function () {
						savedWindowFocusEvent(running_windows, text_span, p_elm);
					};

					editor_button.addEventListener("click", function () {
						edit_state_on = !edit_state_on;

						if (p_elm.lock_pin_system) {
							p_elm.lock_pin_system.input.toggleDisplay(false, false);
						}

						if (edit_state_on) {
							mid_elm.removeEventListener('click', click_handler);
							mid_elm.style.cursor = 'default';
							text_span.style.display = 'none';
							name_input.value = text_span.innerText;
							current_selection = null;
							deactivateItem(document.querySelector('.activated'));
							save_button.style.display = 'inline-block';
							name_input.style.display = 'inline-block';

							if (remove_button) {
								remove_button.style.display = 'inline-block';
							}

							lock_button.style.visibility = 'visible';

						} else {
							mid_elm.addEventListener('click', click_handler);
							mid_elm.style.cursor = 'pointer';
							text_span.style.display = 'inline-block';
							save_button.style.display = 'none';
							name_input.style.display = 'none';

							lock_button.style.visibility = 'hidden';

							if (remove_button) {
								remove_button.style.display = 'none';
							}
						}
					});

					p_elm.className = p_elm.className + ' notrunning';

					name_input.value = text_span.innerText = i;

					mid_elm.addEventListener('click', click_handler);

					left_elm.appendChild(editor_button);
					left_elm.appendChild(save_button);
					mid_elm.appendChild(text_span);
					mid_elm.appendChild(name_input);
					mid_elm.appendChild(remove_button);

					right_elm.appendChild(readonly_button);
					right_elm.appendChild(share_button);

					if (p_elm.window_info.incognito) {
						incognitoBuilder(right_elm);
					}

					container.appendChild(p_elm);

					if (!rendered) {
						window.requestAnimationFrame(function () {
							document.body.style.height = total_count * 23;
							rendered = true;
						});
					}

				})(p_elm);
			}
		}

		if (!search_added) {
			search_box.addEventListener('keyup', function (e) {
				search_added = true;

				if (e.keyCode == 13 || e.keyCode == 38 || e.keyCode == 40) {
					return;
				}

				clearTimeout(search_key_timeout);

				all_elms = all_elms || document.querySelectorAll('.window-item');

				if (search_box.value.trim() == '') {

					for (var i = 0; i < all_elms.length; i++) {
						removeClass(all_elms[i], 'elm-hidden');
					}

					return;
				}

				search_key_timeout = setTimeout(function () {
					var current_set = new RegExp(search_box.value, "i");

					for (var i = 0; i < all_elms.length; i++) {
						if (!all_elms[i].getElementsByClassName('window-text')[0].innerText.match(current_set) ||
							all_elms[i].className.match(notsearchable)
						) {
							all_elms[i].className += ' elm-hidden';
						} else {
							removeClass(all_elms[i], 'elm-hidden');
						}
					}

					visible_elms = document.querySelectorAll('.window-item:not(.elm-hidden)');

					if (visible_elms.length == 1) {
						current_selection = 0;
						visible_elms[current_selection].action_on_hover();
					}
				}, 250);
			});
		}

		document.body.addEventListener('keyup', function (e) {
			body_added = true;

			if (e.keyCode != 13 && e.keyCode != 38 && e.keyCode != 40) {
				return;
			}

			visible_elms = visible_elms || document.querySelectorAll('.window-item:not(.elm-hidden)');

			if (typeof visible_elms[current_selection] == 'undefined') {
				current_selection = null;
			}

			if (e.keyCode == 13 && current_selection != null) {
				(visible_elms[current_selection].getElementsByClassName('window-text'))[0].click();
			}

			// Up arrow
			if (e.keyCode == 38) {
				if (current_selection == null || current_selection == 0) {
					current_selection = (visible_elms.length - 1);
				} else {
					current_selection = current_selection - 1;
				}
			// Down arrow
			} else if (e.keyCode == 40) {
				if (current_selection == null || current_selection == (visible_elms.length - 1)) {
					current_selection = 0;
				} else {
					current_selection = current_selection + 1;
				}
			}

			for (var i = 0; i < visible_elms.length; i++) {
				visible_elms[i].action_on_unhover();
			}

			if (typeof visible_elms[current_selection] != 'undefined') {
				visible_elms[current_selection].action_on_hover();
				visible_elms[current_selection].scrollIntoView();
			}
		});
	});
}

function runningWindowFocusEvent(window_id) {
	chrome.windows.update(window_id, { focused: true });
	window.close();
}

function savedWindowFocusEvent(running_windows, text_span, p_elm) {
	var common_window_create = function () {
		chrome.windows.getCurrent({ populate: true }, function (current_window) {
			if (p_elm.window_info.lock) {
				p_elm.lock_pin_system.lockSystemCallback = function () {
					createWindowBuilder(current_window, p_elm, text_span);
				};

				p_elm.lock_pin_system.input.toggleDisplay(p_elm.lock_pin_system.lockSystemCallback, true);
			} else {
				createWindowBuilder(current_window, p_elm, text_span);
			}
		});
	};

	if (typeof running_windows[text_span.innerText] != 'undefined') {
		chrome.windows.get(running_windows[text_span.innerText], function () {
			if (chrome.runtime.lastError) {
				delete running_windows[text_span.innerText];
				common_window_create();
				return;
			}

			chrome.windows.update(
				parseInt(running_windows[text_span.innerText]),
				{
					focused: true
				}
			);
		});
	} else {
		common_window_create();
	}
};

function editorButton() {
	var editor_button = document.createElement("span");

	editor_button.setAttribute('title', 'Edit');

	editor_button.className = 'glyphicon glyphicon-pencil icon-style edit-icon active-window-toggle';
	editor_button.style.display = 'none';

	return editor_button;
}

function saveButton() {
	var save_button = document.createElement("span");

	save_button.setAttribute('title', 'Save');

	save_button.className = 'glyphicon glyphicon-ok icon-style edit-icon';
	save_button.style.display = 'none';

	return save_button;
}

function windowStyle(active) {
	var active_window = document.createElement("span");

	if (active) {
		active_window.setAttribute('title', 'Active Window');
		active_window.className = "glyphicon glyphicon-flash icon-style";
	} else {
		active_window.setAttribute('title', 'Running Window');
		active_window.className = "glyphicon glyphicon-triangle-right icon-style";
	}

	return active_window;
}

function readOnlyButton(section_name, saved_window_set) {
	var readonly_button = document.createElement("span")

	readonly_button.setAttribute('title', 'Pause Autosave');

	readonly_button.name = section_name;

	readonly_button.updateName = function (name) {
		readonly_button.name = name;
	};

	readonly_button.className = 'glyphicon glyphicon-pause icon-style edit-icon pin-window-toggle action-button';
	readonly_button.style.display = 'inline-block';

	if (saved_window_set['UltiTab-Window'][readonly_button.name] &&
		saved_window_set['UltiTab-Window'][readonly_button.name].read_only) {
		readonly_button.className = readonly_button.className + " button-selected";
		readonly_button.setAttribute('title', 'Resume Autosave');
	}

	readonly_button.addEventListener('click', function () {
		chrome.storage.local.get('UltiTab-Window', function (pin_toggle_sets) {
			pin_toggle_sets['UltiTab-Window'][readonly_button.name].read_only = !pin_toggle_sets['UltiTab-Window'][readonly_button.name].read_only;
			chrome.storage.local.set({
				"UltiTab-Window": pin_toggle_sets['UltiTab-Window']
			},
				function () {
					if (pin_toggle_sets['UltiTab-Window'][readonly_button.name].read_only) {
						var text = 'Autosave paused.';
						readonly_button.setAttribute('title', 'Resume Autosave');
						readonly_button.className = readonly_button.className + " button-selected";
					} else {
						var text = 'Autosave resumed.';
						removeClass(readonly_button, 'button-selected');
						readonly_button.setAttribute('title', 'Pause Autosave');

						chrome.storage.local.get(function (current_data) {

							for (var window_id_key in current_data) {
								if (window_id_key == parseInt(window_id_key)) {
									if (readonly_button.name == current_data[window_id_key].name) {
										var converted_window_id_key = parseInt(window_id_key);

										chrome.windows.get(
											converted_window_id_key,
											{ populate: true },
											function (window_data) {
												if (chrome.runtime.lastError) {
													return;
												}

												current_data['UltiTab-Window'][readonly_button.name].tabs = window_data.tabs;

												chrome.storage.local.set({ 'UltiTab-Window': current_data['UltiTab-Window'] });
											}
										);
									}
								}
							}
						});
					}

					writeBottomText(null, text, false, response_timeout, response_timeout_cleaner);
					buildContextMenu();
				}
			);
		});
	});

	return readonly_button;
}

function removeButton(section_name, running_windows) {
	var remove_item = document.createElement('span');

	remove_item.name = section_name;

	remove_item.updateName = function (name) {
		remove_item.name = name;
	};

	remove_item.setAttribute('title', 'Unwatch');

	remove_item.className = 'glyphicon glyphicon-trash icon-style edit-icon saved-window-toggle';
	remove_item.style.display = 'none';

	remove_item.addEventListener('mouseover', function () {
		remove_item.style.color = 'red';
	});

	remove_item.addEventListener('mouseout', function () {
		remove_item.style.color = 'black';
	});

	remove_item.addEventListener('click', function () {
		chrome.storage.local.get('UltiTab-Window', function (delete_sets) {
			if (typeof running_windows[remove_item.name] != 'undefined') {
				chrome.storage.local.remove(running_windows[remove_item.name].toString());
				delete running_windows[remove_item.name];
			}

			delete delete_sets['UltiTab-Window'][remove_item.name];

			chrome.storage.local.set({
				"UltiTab-Window": delete_sets['UltiTab-Window']
			},
				function () {
					buildContextMenu();
					getWindowList(buildWindowlistUI);
				}
			);
		});
	}, true);

	return remove_item;
}

function shareButton(parent, shared) {
	var share_button = document.createElement("span");
	share_button.className = 'glyphicon glyphicon-share-alt icon-style edit-icon action-button';

	share_button.setAttribute("title", "Share Window");

	if (shared) {
		share_button.style.color = 'black';
	}

	share_button.addEventListener("click", function () {
		chrome.windows.create({url: "chrome-extension://" + chrome.runtime.id + "/options.html"});
	});

	return share_button;
}

function nameInput(placeholder_name) {
	var name_input = document.createElement("input");

	name_input.type = 'text';
	name_input.placeholder = placeholder_name;
	name_input.className = 'baseline-textbox';

	return name_input;
}

function createColumn(class_name) {
	var column_elm = document.createElement('div');
	column_elm.className = class_name;
	return column_elm;
}

function createWindowItemFromList(
	container,
	windowList,
	running_windows,
	sets,
	display_class,
	bottom_text
) {
	return new Promise(function (resolve, reject) {

		var window_id = windowList.id.toString(),
			p_elm = windowItemContainerCreate();

		chrome.storage.local.get(window_id, function (data) {
			(function (p_elm) {
				var edit_state_on = false,
					incognito = windowList.incognito,
					text_span = windowItemCreate('', windowList.tabs.length),
					editor_button = editorButton(),
					save_button = saveButton(),
					name_input = nameInput('Unsaved window #' + window_id),
					readonly_button = null,
					lock_button = null,
					remove_button = null,
					share_button = null,
					window_style = windowStyle((display_class == "focused-on")),
					left_elm = createColumn('col-first'),
					mid_elm = createColumn('col-middle'),
					right_elm = createColumn('col-end');

				p_elm.window_info = windowList;

				// Next elements for containing interactions
				p_elm.appendChild(left_elm);
				p_elm.appendChild(mid_elm);
				p_elm.appendChild(right_elm);

				// Finishing left elem
				left_elm.appendChild(window_style);
				left_elm.appendChild(editor_button);
				left_elm.appendChild(save_button);

				text_span.id = 'window_id_' + window_id;

				mid_elm.appendChild(text_span);
				mid_elm.appendChild(name_input);

				name_input.addEventListener('keyup', function (event) {
					if (event.keyCode == 13) {
						saveWindowEntry(
							p_elm,
							parseInt(window_id),
							name_input.value,
							text_span,
							editor_button,
							bottom_text,
							sets,
							function (saved_window_info) {
								if (readonly_button) {
									readonly_button.updateName(name_input.value);
									remove_button.updateName(name_input.value);
									lock_button.updateName(lock_button.value);
								} else {
									readonly_button = readOnlyButton(name_input.value, saved_window_info);
									remove_button = removeButton(name_input.value, running_windows);
									lock_button = lockBuilder(p_elm, text_span, mid_elm, mid_elm, right_elm, name_input, remove_button);
									share_button = shareButton(p_elm, (typeof saved_window_info['UltiTab-Window'][name_input.value].saved_window_url != 'undefined' ? true : false));
									lock_button.style.visibility = 'visible';
									remove_button.style.display = 'inline-block';
									right_elm.appendChild(lock_button);
									right_elm.appendChild(readonly_button);
									right_elm.appendChild(share_button);
									mid_elm.appendChild(remove_button);
								}
							},
							function () {
								writeBottomText(bottom_text, 'Name already used.', true, response_timeout, response_timeout_cleaner);
							},
							running_windows
						);
					}
				});

				save_button.addEventListener("click", function () {
					saveWindowEntry(
						p_elm,
						parseInt(window_id),
						name_input.value,
						text_span,
						editor_button,
						bottom_text,
						sets,
						function (saved_window_info) {
							if (readonly_button) {
								name_input.placeholder = name_input.value;
								readonly_button.updateName(name_input.value);
								remove_button.updateName(name_input.value);
								lock_button.updateName(lock_button.value);
							} else {
								readonly_button = readOnlyButton(name_input.value, saved_window_info);
								remove_button = removeButton(name_input.value, running_windows);
								lock_button = lockBuilder(p_elm, text_span, mid_elm, mid_elm, right_elm, name_input, remove_button);
								share_button = shareButton(p_elm, (typeof saved_window_info['UltiTab-Window'][name_input.value].saved_window_url != 'undefined' ? true : false));
								lock_button.style.visibility = 'visible';
								remove_button.style.display = 'inline-block';
								right_elm.appendChild(lock_button);
								right_elm.appendChild(readonly_button);
								right_elm.appendChild(share_button);
								mid_elm.appendChild(remove_button);
							}
						},
						function () {
							writeBottomText(bottom_text, 'Name already used.', true, response_timeout, response_timeout_cleaner);
						},
						running_windows
					);
				});

				var click_handler = function () {
					runningWindowFocusEvent(p_elm.window_info.id);
				};

				editor_button.addEventListener("click", function () {
					edit_state_on = !edit_state_on;

					if (p_elm.lock_pin_system) {
						p_elm.lock_pin_system.input.toggleDisplay(false, false);
					}

					if (edit_state_on) {
						mid_elm.removeEventListener('click', click_handler);
						mid_elm.style.cursor = 'default';
						text_span.style.display = 'none';
						name_input.value = (running_windows[text_span.innerText] ? text_span.innerText : '');
						current_selection = null;
						deactivateItem(document.querySelector('.activated'));
						save_button.style.display = 'inline-block';
						name_input.style.display = 'inline-block';

						if (lock_button) {
							lock_button.style.visibility = 'visible';
						}

						if (remove_button) {
							remove_button.style.display = 'inline-block';
						}

					} else {
						mid_elm.addEventListener('click', click_handler);
						mid_elm.style.cursor = 'pointer';
						text_span.style.display = 'inline-block';
						save_button.style.display = 'none';
						name_input.style.display = 'none';

						if (lock_button) {
							lock_button.style.visibility = 'hidden';
						}

						if (remove_button) {
							remove_button.style.display = 'none';
						}
					}
				});

				mid_elm.addEventListener('click', click_handler);

				if (typeof data[window_id] != 'undefined' &&
					typeof data[window_id].name != 'undefined'
				) {
					name_input.placeholder = data[window_id].name;
					running_windows[data[window_id].name] = window_id;
					text_span.innerText = name_input.value = data[window_id].name;

					if (typeof sets['UltiTab-Window'][data[window_id].name] != 'undefined') {
						p_elm.window_info.position = sets['UltiTab-Window'][data[window_id].name].position || {
							top: windowList.top,
							left: windowList.left,
							width: windowList.width,
							height: windowList.height,
							state: windowList.state
						};

						p_elm.window_info.lock = sets['UltiTab-Window'][data[window_id].name].lock || false;
					} else {
						p_elm.window_info.position = {
							top: windowList.top,
							left: windowList.left,
							width: windowList.width,
							height: windowList.height,
							state: windowList.state
						};
					}


					var readonly_button = readOnlyButton(data[window_id].name, sets),
						lock = (sets['UltiTab-Window'][data[window_id].name] ? sets['UltiTab-Window'][data[window_id].name].lock : false),
						remove_button = removeButton(data[window_id].name, running_windows),
						lock_button = lockBuilder(p_elm, text_span, mid_elm, mid_elm, right_elm, name_input, remove_button),
						share_button = shareButton(p_elm, (typeof sets['UltiTab-Window'][data[window_id].name].saved_window_url != 'undefined' ? true : false));

					mid_elm.appendChild(remove_button);

					right_elm.appendChild(lock_button);
					right_elm.appendChild(readonly_button);
					right_elm.appendChild(share_button);
				} else {
					removeClass(p_elm, 'searchable');
					p_elm.className += ' notsearchable';

					text_span.innerText = 'Unsaved window #' + window_id;

					p_elm.window_info.position = {
						top: windowList.top,
						left: windowList.left,
						width: windowList.width,
						height: windowList.height,
						state: windowList.state
					};
				}

				p_elm.className = p_elm.className + ' ' + display_class;

				if (incognito) {
					incognitoBuilder(right_elm);
				}

				resolve();
			})(p_elm);
		});

		container.appendChild(p_elm);
	});
}

function clearStorage() {
	chrome.storage.local.clear();
}

function saveWindowEntry(
	p_elm,
	window_id,
	input_value,
	text_span,
	editor_button,
	bottom_text,
	sets,
	callback,
	error_callback,
	running_windows
) {
	if (input_value.trim() == '') {
		return;
	}

	chrome.tabs.query({
		windowId: window_id
	},
		function (tabs) {
			var elm = document.getElementById('window_id_' + window_id);

			setRunningWindowInfo(
				window_id,
				input_value,
				elm.innerText,
				tabs,
				p_elm.window_info.incognito,
				sets,
				function (new_sets) {
					text_span.innerText = input_value;
					writeBottomText(bottom_text, 'Name saved.', false, response_timeout, response_timeout_cleaner);
					callback(new_sets);
				},
				error_callback,
				running_windows
			);
		});
}

function windowItemContainerCreate() {
	var window_item = document.createElement('p');
	window_item.className = 'window-item searchable';

	window_item.action_on_hover = function () {
		deactivateItem(document.querySelector('.activated'));
		if (window_item.className.indexOf(' activated') === -1) {
			window_item.className = window_item.className + ' activated';
		}
	};

	window_item.action_on_unhover = function () {
		deactivateItem(window_item);
	};

	window_item.addEventListener('mouseover', window_item.action_on_hover);

	window_item.addEventListener('mouseout', window_item.action_on_unhover);

	return window_item;
}

function windowItemCreate(text, tab_number) {
	var window_item_text = document.createElement('span');
	window_item_text.style.cursor = 'pointer';
	window_item_text.className = 'window-text';
	window_item_text.innerText = text;
	window_item_text.setAttribute("title", tab_number + " Tabs");
	return window_item_text;
}

function incognitoBuilder(parent) {
	var icon_span = document.createElement('span');
	icon_span.className = 'glyphicon glyphicon-sunglasses icon-style';
	parent.appendChild(icon_span)
}

function lockButton(section_name, locked, parent) {
	var icon_span = document.createElement("span");

	icon_span.setAttribute('title', 'Lock Window');

	icon_span.name = section_name;

	icon_span.updateName = function (name) {
		icon_span.name = name;
	};

	icon_span.className = 'glyphicon glyphicon-lock icon-style edit-icon action-button';
	icon_span.style.visibility = 'hidden';

	if (locked) {
		icon_span.className = icon_span.className + ' button-selected';
		icon_span.setAttribute('title', 'Unlock Window');
	}

	icon_span.addEventListener('click', function () {
		chrome.storage.local.get('lock_code', function (lock_code) {
			chrome.storage.local.get('UltiTab-Window', function (lock_toggle_sets) {

				var toggle_lock_resolver = function () {
					lock_toggle_sets['UltiTab-Window'][icon_span.name].lock = !lock_toggle_sets['UltiTab-Window'][icon_span.name].lock;

					chrome.storage.local.set(
						{
							"UltiTab-Window": lock_toggle_sets['UltiTab-Window']
						},
						function () {
							if (typeof lock_code.lock_code == 'undefined') {
								chrome.windows.create({url: "chrome-extension://" + chrome.runtime.id + "/options.html"});
							} else {
								if (lock_toggle_sets['UltiTab-Window'][icon_span.name].lock) {
									var text = 'Window locked.';
									parent.window_info.lock = true;
									icon_span.className = icon_span.className + ' button-selected';
									icon_span.setAttribute('title', 'Unlock Window');
								} else {
									var text = 'Window unlocked.';
									icon_span.setAttribute('title', 'Unlock Window');
									parent.window_info.lock = false;
									removeClass(icon_span, 'button-selected');
									icon_span.setAttribute('title', 'Lock Window');
								}

								parent.lock_pin_system.input.toggleDisplay(function () { }, false);
								writeBottomText(null, text, false, response_timeout, response_timeout_cleaner);
							}
						}
					);
				};

				if (lock_toggle_sets['UltiTab-Window'][icon_span.name].lock &&
					typeof parent.lock_pin_system != 'undefined'
				) {
					parent.lock_pin_system.input.toggleDisplay(toggle_lock_resolver, true);
				} else {
					toggle_lock_resolver();
				}
			});
		});
	});

	return icon_span;
}

function lockBuilder(parent, window_text, cancel_button_container, lock_input_container, lock_icon_container, name_input, remove_button) {
	var icon_span = lockButton(window_text.innerText, parent.window_info.lock, parent),
		lock_system_pin_input = document.createElement('input'),
		cancel_span = document.createElement('span');

	cancel_span.className = 'glyphicon glyphicon-ban-circle icon-style edit-icon cancel-window-toggle';

	cancel_span.setAttribute('title', 'Cancel unlock');

	lock_input_container.appendChild(lock_system_pin_input);
	cancel_button_container.appendChild(cancel_span);
	lock_icon_container.appendChild(icon_span);

	lock_system_pin_input.type = 'password';
	lock_system_pin_input.setAttribute('maxlength', 4);
	lock_system_pin_input.placeholder = 'Enter Pin';
	lock_system_pin_input.style.display = 'none';
	lock_system_pin_input.className = 'baseline-textbox';

	lock_system_pin_input.addEventListener('keyup', function (e) {
		if (e.keyCode == 13) {
			lock_system_pin_input.style.border = 'thin solid black';

			chrome.storage.local.get('lock_code', function (lock_code) {
				if (typeof lock_code.lock_code == 'undefined') {
					chrome.windows.create({url: "chrome-extension://" + chrome.runtime.id + "/options.html"});
					return;
				}

				if (lock_code.lock_code == md5(lock_system_pin_input.value.trim())) {
					lock_system_pin_input.resolverCallback();
				} else {
					lock_system_pin_input.value = '';
					lock_system_pin_input.style.border = 'thin solid red';
					lock_system_pin_input.placeholder = 'Incorrect Pin.';
				}
			});
		}
	});

	lock_system_pin_input.toggle = false;

	lock_system_pin_input.toggleDisplay = function (toggleResolver, forced_toggle) {
		lock_system_pin_input.toggle = forced_toggle;

		if (toggleResolver !== false) {
			lock_system_pin_input.resolverCallback = toggleResolver;
		}

		if (lock_system_pin_input.toggle) {
			window_text.style.display = 'none';
			name_input.style.display = 'none';
			remove_button.style.display = 'none';
			lock_system_pin_input.style.display = 'inline-block';
			lock_system_pin_input.focus();
			cancel_span.style.display = 'inline-block';
		} else {
			name_input.style.display = 'inline-block';
			remove_button.style.display = 'inline-block';
			lock_system_pin_input.style.display = 'none';
			cancel_span.style.display = 'none';
			lock_system_pin_input.value = '';
		}
	};

	cancel_span.style.display = 'none';

	cancel_span.addEventListener('mouseover', function () {
		cancel_span.style.color = 'red';
	});

	cancel_span.addEventListener('mouseout', function () {
		cancel_span.style.color = 'black';
	});

	cancel_span.addEventListener('click', function () {
		lock_system_pin_input.toggleDisplay(function () { }, false);
	});

	parent.lock_pin_system = { input: lock_system_pin_input };

	return icon_span;
}

function deactivateItem(item) {
	if (item == null) {
		return;
	}

	removeClass(item, 'activated');
}

function getWindowList(callback) {
	var allWindows = new Promise(function (resolve, reject) {
		chrome.windows.getAll({ populate: true }, function (windowList) {
			resolve(windowList);
		});
	});
	var currentWindow = new Promise(function (resolve, reject) {
		chrome.windows.getCurrent({ populate: true }, function (current_window) {
			resolve(current_window);
		});
	});
	var savedWindows = new Promise(function (resolve, reject) {
		chrome.storage.local.get('UltiTab-Window', function (sets) {
			resolve(sets);
		});
	});

	Promise.all([currentWindow, allWindows, savedWindows]).then(function (chromeResults) {
		callback(chromeResults[0], chromeResults[1], chromeResults[2]);
	});
}

function createWindowBuilder(current_window, window_item, window_item_text) {
	chrome.runtime.sendMessage({
		type: "create_window",
		content: {
			current_window: current_window,
			window_item: window_item,
			name: window_item_text.innerText
		}
	}, function(response) {
		window.close();
	});
}

Raven.context(function () {
	getWindowList(buildWindowlistUI);
});